SystemOrganization addCategory: #'Phidget-VMMaker-Plugins'!
SystemOrganization addCategory: #Phidget!
SystemOrganization addCategory: #'Phidget-Core'!
SystemOrganization addCategory: #'Phidget-Lab'!
SystemOrganization addCategory: #'Phidget-Tests'!
SystemOrganization addCategory: #'Phidget-EToys'!

!IconicButton methodsFor: '*Phidget' stamp: 'pt 5/7/2009 11:45'!
initializeWithThumbnail: aThumbnail withLabel: aLabel andColor: aColor andSend: aSelector to: aReceiver 	with: aPhidget
	"Initialize the receiver to show aThumbnail on its face, giving it the label supplied and arranging for it, when the button goes down on it, to obtain a new morph by sending the supplied selector to the supplied receiver"

	| labeledItem nonTranslucent |
	nonTranslucent := aColor asNontranslucentColor.
	labeledItem := AlignmentMorph newColumn.
	labeledItem color: nonTranslucent.
	labeledItem borderWidth: 0.
	labeledItem
		layoutInset: 4@0;
		cellPositioning: #center.
	labeledItem addMorph: aThumbnail.
	labeledItem addMorphBack: (Morph new extent: (4@4)) beTransparent.
	labeledItem addMorphBack: (TextMorph new
		backgroundColor: nonTranslucent;
		contentsAsIs: aLabel;
		beAllFont: Preferences standardEToysFont;
		centered).

	self
		beTransparent;
		labelGraphic: ((labeledItem imageForm: 32 backgroundColor: nonTranslucent forRectangle: labeledItem fullBounds) replaceColor: nonTranslucent withColor: Color transparent);
		borderWidth: 0;
		target: aReceiver;
		actionSelector: #launchPartVia:label:phidget:;
		arguments: {aSelector. aLabel. aPhidget};
		actWhen: #buttonDown.

	self stationarySetup.! !

Object subclass: #PhdCrPhidget
	instanceVariableNames: 'semaphores processBlocks opened'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Core'!

!PhdCrPhidget commentStamp: 'pt 5/3/2009 14:26' prior: 0!
A PhdCrPhidget is an abstract class that provides some primitives and methods for Phidget support. Please read the comments of my subclasses to find more information how to use Phidgets in Squeak.

Instance Variables
	opened:			<Boolean>
	processBlocks:	<Dictionary>
	semaphores:		<Dictionary>

opened
	- true, if the Phidget was opened - Squeak is able to communicate with the hardware.
	- false, if the Phidgets was not opened or was opened and then closed.

semaphores
	- A Dictionary with entries of the following form: #aSymbol -> aSemaphore.
	#aSymbol may be #Attach, #Detach, #Error for Phidgets in general. Other symbols may excist for phidget specific support.
	This dictionary is used to handle the callbacks from the Phidget C API.
	These semaphores can be registered for certain events (like the attach event) - so that the C API signals is if the event occours. These semaphores are used in the callbackProcess method of this class.

processBlocks
	- A Dictionary with entries of the following form: #aSymbol -> aBlock.
	The symbols are equivalent to the symbols used in the semaphores dictionary.
	This dictionary is used to handle the callbacks from the Phidget C API.
	Each time a semaphore from the semaphores dictionary was signaled from the C API, the corresponding block vom this dictionary will be evaluated by the callbackProcess method of this class.
	!

PhdCrPhidget subclass: #PhdCrInterfaceKit
	instanceVariableNames: 'ifKit'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Core'!

!PhdCrInterfaceKit commentStamp: 'pt 5/3/2009 15:04' prior: 0!
A PhdCrInterfaceKit is a representation of a Phidget called 'Interface Kit' in Squeak.
To use the Interface Kit, create a new instance of this class and open a connection to the hardware board with:
	ifKit := PhdCrInterfaceKit new.
	ifKit openAny.
A block can be set to be evaluated every time the hardware board was attached, detached, an error occoured, a sensor value was changed, an input value was changed or an output value was changed. For example:
	ifKit
		onAttachDo: [ Transcript show:'servo was attached!!;cr' ];
		onDetachDo: [ Transcript show:'servo was detached!!;cr' ];
		onSensorChangeDo: [ Transcript show:'A sensor changed it's value!!' ].

Depending on which hardware is attached it is possible to attach one or more analog sensors (, digital inputs or digital outputs) to the board. To get the number of available sensors send 'sensorCount' ('inputCount', 'outputCount'). Every sensor can be accessed by an index adressing it. Valid indices are greater or equal than 1 and less or equal than the number of available sensors (sensorCount).
To get the value of a sensor, call 'sensorValue: index'. The value will be a number between 0 and 1000.
The input and output values are boolean and may be read by: 'outputValue: index' or 'inputValue: index'.
It is possible to set an output value to true/false with 'setOutputValue: aBoolean at: index'.

IMPORTANT:
After using the Interface Kit it is important to close the connection to the hardware by sending 'close':
	ifKit close.

NOTE:
The sensor changeBlock will be evaluated only, it the change is greater than a number called sensitivity. Example: A light sensor probably changes its value continuously. We want to know greater changes only, so we set the sensitivity to 50. So the sensor change block will be evaluated only if the change is greater than 50. To set the sensitivity, call 'sensitivity: aNumber to: index'.


Instance Variables
	ifKit:		<ByteArray>

ifKit
	- a ByteArray that contains the Interface Kit handle for the C API. Every primitive (except the one where the handle is created) needs this ByteArray to let the C API know about which Interface Kit it is currently talking.
!

!PhdCrInterfaceKit methodsFor: 'as yet unclassified' stamp: 'pt 5/3/2009 20:05'!
attached
	"Returns true if the Interface Kit is attached."
	
	( ( self primAttached: ifKit ) == 1 ) ifTrue: [ ^ true ].
	^ false! !

!PhdCrInterfaceKit methodsFor: 'as yet unclassified' stamp: 'pt 5/2/2009 22:47'!
close
	"Close the Interface Kit."

	opened ifFalse: [^ self]. "already closed"
	self primClose: ifKit.
	opened := false.
	self unregisterSemaphores.! !

!PhdCrInterfaceKit methodsFor: 'as yet unclassified' stamp: 'pt 5/2/2009 20:41'!
initialize
	"Initialise the callbacks that are not already initialised in PhdCrPhidget.
	Then initialise the Interface Kit handle, which is used to call the primitives."

	super initialize.
	semaphores
		add: #SensorChange->Semaphore new;
		add: #InputChange->Semaphore new;
		add: #OutputChange->Semaphore new.
		
	processBlocks
		add: #SensorChange->[];
		add: #InputChange->[];
		add: #OutputChange->[].
		
	ifKit := self primCreate.! !

!PhdCrInterfaceKit methodsFor: 'as yet unclassified' stamp: 'pt 5/7/2009 18:42'!
inputCount
	"Returns the number of inputs that may be attached to the Interface Kit."
	
	self attached
		ifTrue: [ ^ self primNumInputs: ifKit ].
	^ 0! !

!PhdCrInterfaceKit methodsFor: 'as yet unclassified' stamp: 'pt 5/3/2009 11:28'!
inputIndexIsInRange: index
	"Returns true if 1<=index<=sensorCount.
	Every time a primitive is used that needs the input index, this method should be used to check if the index is in the correct range."
	
	index isInteger ifFalse: [ ^ false ].
	^ ( 1 <= index ) and: [ index <= self inputCount ]! !

!PhdCrInterfaceKit methodsFor: 'as yet unclassified' stamp: 'pt 5/3/2009 13:18'!
inputValue: inputIndex
	"Gets the output value (true or false) at the digital output with the given index."

	( self inputIndexIsInRange: inputIndex ) ifFalse: [ ^ nil ].
	( ( self primInputValue: ifKit at: inputIndex - 1 ) == 1 ) ifTrue: [ ^ true ].
	^ false! !

!PhdCrInterfaceKit methodsFor: 'as yet unclassified' stamp: 'pt 5/2/2009 22:58'!
onInputChangeDo: aBlock
	"The aBlock will be evaluated if an input changed it's state."

	processBlocks at: #InputChange put: aBlock.! !

!PhdCrInterfaceKit methodsFor: 'as yet unclassified' stamp: 'pt 5/2/2009 22:59'!
onOutputChangeDo: aBlock
	"The aBlock will be evaluated if an output changed it's state."

	processBlocks at: #OutputChange put: aBlock.! !

!PhdCrInterfaceKit methodsFor: 'as yet unclassified' stamp: 'pt 5/2/2009 22:59'!
onSensorChangeDo: aBlock
	"The aBlock will be evaluated if a sensor changed it's value."

	processBlocks at: #SensorChange put: aBlock.! !

!PhdCrInterfaceKit methodsFor: 'as yet unclassified' stamp: 'pt 5/2/2009 23:15'!
openAny
	"Opens any Interface Kit."
	
	ifKit ifNil: [ self error: 'Could not open InterfaceKit!!'. ^ self ].
	self registerSemaphores.
	self primOpenAny: ifKit.
	opened := true.
	[ self callbackProcess: #Attach ] fork.
	[ self callbackProcess: #Detach ] fork.
	[ self callbackProcess: #Error ] fork.
	[ self callbackProcess: #SensorChange ] fork.
	[ self callbackProcess: #InputChange ] fork.
	[ self callbackProcess: #OutputChange ] fork.! !

!PhdCrInterfaceKit methodsFor: 'as yet unclassified' stamp: 'pt 5/2/2009 23:17'!
openSpecefic: serialNumber 
	"Opens the Interface Kit with the given serial number."
	ifKit ifNil: 
		[ self error: 'Could not open InterfaceKit!!'.
		^ self ].
	self registerSemaphores.
	self 
		primOpen: ifKit
		withSerial: serialNumber.
	opened := true.
	[ self callbackProcess: #Attach ] fork.
	[ self callbackProcess: #Detach ] fork.
	[ self callbackProcess: #Error ] fork.
	[ self callbackProcess: #SensorChange ] fork.
	[ self callbackProcess: #InputChange ] fork.
	[ self callbackProcess: #OutputChange ] fork! !

!PhdCrInterfaceKit methodsFor: 'as yet unclassified' stamp: 'pt 5/7/2009 18:42'!
outputCount
	"Returns the number of outputs that may be attached to the Interface Kit."
	
	self attached
		ifTrue: [ ^ self primNumOutputs: ifKit ].
	^ 0! !

!PhdCrInterfaceKit methodsFor: 'as yet unclassified' stamp: 'pt 5/3/2009 11:27'!
outputIndexIsInRange: index
	"Returns true if 1<=index<=sensorCount.
	Every time a primitive is used that needs the output index, this method should be used to check if the index is in the correct range."
	
	index isInteger ifFalse: [ ^ false ].
	^ ( 1 <= index ) and: [ index <= self outputCount ]! !

!PhdCrInterfaceKit methodsFor: 'as yet unclassified' stamp: 'pt 5/3/2009 13:16'!
outputValue: outputIndex
	"Gets the output value (true or false) at the digital output with the given index."

	( self outputIndexIsInRange: outputIndex ) ifFalse: [ ^ nil ].
	( ( self primOutputValue: ifKit at: outputIndex - 1 ) == 1 ) ifTrue: [ ^ true ].
	^ false! !

!PhdCrInterfaceKit methodsFor: 'primitives' stamp: 'pt 5/2/2009 23:01'!
primAttached: phid
	"Return 1 if the Interface Kit is attached - otherweise return 0."
	
	<primitive: 'primIfKitAttached' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrInterfaceKit methodsFor: 'primitives' stamp: 'pt 5/2/2009 23:02'!
primClose: phid
	"Close the Interface Kit."
	
	<primitive: 'primIfKitClose' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrInterfaceKit methodsFor: 'primitives' stamp: 'pt 5/2/2009 23:02'!
primCreate
	"Reset the Interface Kit (Semaphores and Handle)."

	<primitive: 'primIfKitCreate' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrInterfaceKit methodsFor: 'primitives' stamp: 'pt 5/3/2009 00:05'!
primGetSensorChangeTrigger: phid at: sensorIndex
	"Gets the sensor change trigger. A sensor change will be registered if the change is higher than this number only."

	<primitive: 'primIfKitGetSensorChangeTrigger' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrInterfaceKit methodsFor: 'primitives' stamp: 'pt 5/2/2009 23:37'!
primInputValue: phid at: sensorIndex
	"Gets the input value (0, 1) at the input with the given index."

	<primitive: 'primIfKitInputValue' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrInterfaceKit methodsFor: 'primitives' stamp: 'pt 5/6/2009 17:06'!
primNumInputs: phid
	"Gets the number of inputs."

	<primitive: 'primIfKitNumInput' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrInterfaceKit methodsFor: 'primitives' stamp: 'pt 5/6/2009 17:06'!
primNumOutputs: phid
	"Gets the number of outputs."

	<primitive: 'primIfKitNumOutput' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrInterfaceKit methodsFor: 'primitives' stamp: 'pt 5/2/2009 23:06'!
primNumSensors: phid
	"Gets the number of sensors."

	<primitive: 'primIfKitNumSensors' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrInterfaceKit methodsFor: 'primitives' stamp: 'lk 5/5/2009 17:14'!
primOnAttached: phid semaphoreIndex: semaIndex
	"registers the given Semaphore to be signaled when the Interface Kit becomes attached."
	
	<primitive: 'primIfKitOnAttached' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrInterfaceKit methodsFor: 'primitives' stamp: 'lk 5/5/2009 17:14'!
primOnDetached: phid semaphoreIndex: semaIndex
	"registers the given Semaphore to be signaled when the Interface Kit becomes detached."
	
	<primitive: 'primIfKitOnDetached' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrInterfaceKit methodsFor: 'primitives' stamp: 'lk 5/5/2009 17:14'!
primOnError: phid semaphoreIndex: semaIndex
	"registers the given Semaphore to be signaled when a an Error occours. Use primLastError to get the last error code"
	
	<primitive: 'primIfKitOnError' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrInterfaceKit methodsFor: 'primitives' stamp: 'lk 5/5/2009 17:14'!
primOnInputChanged: phid semaphoreIndex: semaIndex
	"registers the given Semaphore to be signaled when an InputValue changed."
	
	<primitive: 'primIfKitOnInputChanged' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrInterfaceKit methodsFor: 'primitives' stamp: 'lk 5/5/2009 17:15'!
primOnOutputChanged: phid semaphoreIndex: semaIndex
	"registers the given Semaphore to be signaled when an OutputValue changed."
	
	<primitive: 'primIfKitOnOutputChanged' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrInterfaceKit methodsFor: 'primitives' stamp: 'lk 5/5/2009 17:15'!
primOnSensorChanged: phid semaphoreIndex: semaIndex
	"registers the given Semaphore to be signaled when a SensorValue changed."
	
	<primitive: 'primIfKitOnSensorChanged' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrInterfaceKit methodsFor: 'primitives' stamp: 'pt 5/2/2009 23:11'!
primOpen: phid withSerial: serial
	"Opens the Interface Kit with the given serial number"

	<primitive: 'primIfKitOpen' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrInterfaceKit methodsFor: 'primitives' stamp: 'pt 5/2/2009 23:11'!
primOpenAny: phid
	"Opens any available Interface Kit"

	<primitive: 'primIfKitOpenAny' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrInterfaceKit methodsFor: 'primitives' stamp: 'pt 5/2/2009 23:37'!
primOutputValue: phid at: sensorIndex
	"Gets the output value (0, 1) at the output with the given index."

	<primitive: 'primIfKitOutputValue' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrInterfaceKit methodsFor: 'primitives' stamp: 'pt 5/2/2009 23:12'!
primSensorValue: phid at: sensorIndex
	"Gets the sensor value (0-1000) at the analog input with the given index."

	<primitive: 'primIfKitSensorValue' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrInterfaceKit methodsFor: 'primitives' stamp: 'pt 5/6/2009 17:12'!
primSetOutput: phid at: sensorIndex to: aNumber
	"Sets the output to  0/1 (which means ON/OFF) at the output with the given index."

	<primitive: 'primIfKitSetOutput' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrInterfaceKit methodsFor: 'primitives' stamp: 'pt 5/3/2009 00:02'!
primSetSensorChangeTrigger: phid at: sensorIndex to: aNumber
	"Sets the sensor change trigger to  aNumber (0-1000). A sensor change will be registered if the change is higher than aNumber only."

	<primitive: 'primIfKitSetSensorChangeTrigger' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrInterfaceKit methodsFor: 'as yet unclassified' stamp: 'lk 5/7/2009 13:28'!
registerSemaphores
	"Register the semaphores, so that the PhidgetPlugin can signal them."

	| semaIndex |
	semaIndex := Smalltalk registerExternalObject: ( semaphores at:#SensorChange ).
	self primOnSensorChanged: ifKit semaphoreIndex: semaIndex.
	semaIndex := Smalltalk registerExternalObject: ( semaphores at:#InputChange ).
	self primOnInputChanged: ifKit semaphoreIndex: semaIndex.
	semaIndex := Smalltalk registerExternalObject: ( semaphores at:#OutputChange ).
	self primOnOutputChanged: ifKit semaphoreIndex: semaIndex.
	semaIndex := Smalltalk registerExternalObject: ( semaphores at:#Attach ).
	self primOnAttached: ifKit semaphoreIndex: semaIndex.
	semaIndex := Smalltalk registerExternalObject: ( semaphores at:#Detach ).
	self primOnDetached: ifKit semaphoreIndex: semaIndex.
	semaIndex := Smalltalk registerExternalObject: ( semaphores at:#Error ).
	self primOnError: ifKit semaphoreIndex: semaIndex.! !

!PhdCrInterfaceKit methodsFor: 'as yet unclassified' stamp: 'pt 5/3/2009 14:59'!
sensitivity: sensitivity at: index 
	"Sets the input sensitivity. The onSensorChange block will be evaluated only, if the change of the sensor value is greater than the input sensitivity."
	(self sensorIndexIsInRange: index) ifFalse: [ ^ false ].
	^ self 
		primSetSensorChangeTrigger: ifKit
		at: index - 1
		to: sensitivity! !

!PhdCrInterfaceKit methodsFor: 'as yet unclassified' stamp: 'pt 5/3/2009 15:00'!
sensitivityAt: index 
	"Returns the input sensitivity. The onSensorChange block will be evaluated only, if the change of the sensor value is greater than the input sensitivity."
	(self sensorIndexIsInRange: index) ifFalse: [ ^ false ].
	^ self 
		primGetSensorChangeTrigger: ifKit
		at: index - 1! !

!PhdCrInterfaceKit methodsFor: 'as yet unclassified' stamp: 'pt 5/7/2009 18:41'!
sensorCount
	"Returns the number of sensors that may be attached to the Interface Kit."
	self attached
		ifTrue: [ ^ self primNumSensors: ifKit ].
	 ^ 0! !

!PhdCrInterfaceKit methodsFor: 'as yet unclassified' stamp: 'pt 5/3/2009 11:27'!
sensorIndexIsInRange: index
	"Returns true if 1<=index<=sensorCount.
	Every time a primitive is used that needs the sensor index, this method should be used to check if the index is in the correct range."
	
	index isInteger ifFalse: [ ^ false ].
	^ ( 1 <= index ) and: [ index <= self sensorCount ]! !

!PhdCrInterfaceKit methodsFor: 'as yet unclassified' stamp: 'pt 5/3/2009 11:29'!
sensorValue: sensorIndex
	"Gets the sensor value (0-1000) at the analog input with the given index."

	(self sensorIndexIsInRange: sensorIndex) ifFalse: [ ^ nil ].
	^ self primSensorValue: ifKit at: sensorIndex - 1! !

!PhdCrInterfaceKit methodsFor: 'as yet unclassified' stamp: 'pt 5/6/2009 17:32'!
setOutputValue: aBoolean at: outputIndex
	"Sets the output value (true or false) at the digital output with the given index."

	| anInteger |
	( self outputIndexIsInRange: outputIndex ) ifFalse: [ ^ nil ].
	aBoolean
		ifTrue: [ anInteger := 1 ]
		ifFalse: [ anInteger := 0 ].
	^ self primSetOutput: ifKit at: outputIndex-1 to: anInteger.! !

!PhdCrPhidget methodsFor: 'as yet unclassified' stamp: 'pt 5/2/2009 21:51'!
callbackProcess: aSymbol
	"This method will be forked as an extra process for every callback that a phidget needs.
	aSymbol can be #Attach, #Detach, #Error for every phidget. Other phidget-dependent symbols have to be set (like in this initialise method) in the phidgets initialise method"

	( semaphores at: aSymbol ) wait.
	[opened] whileTrue: [
		( processBlocks at: aSymbol ) value.
		( semaphores at: aSymbol ) wait ].! !

!PhdCrPhidget methodsFor: 'as yet unclassified' stamp: 'pt 5/2/2009 21:52'!
initialize
	"Initialise the callbacks that are used in every Phidget.
	Every subclass should call super initialise or initialise the callbacks by their own."

	super initialize.
	semaphores := Dictionary new.
	semaphores
		add: #Attach->Semaphore new;
		add: #Detach->Semaphore new;
		add: #Error->Semaphore new.
		
	processBlocks := Dictionary new.
	processBlocks
		add: #Attach->[];
		add: #Detach->[];
		add: #Error->[].
		
	opened := false.! !

!PhdCrPhidget methodsFor: 'as yet unclassified' stamp: 'pt 5/2/2009 22:10'!
lastError
	"Returns the error code of the last error.
	Descriptions of the codes may be found on www.phidgets.com or phidget21.h on line 2250 ff."

	^ self primLastError! !

!PhdCrPhidget methodsFor: 'as yet unclassified' stamp: 'pt 5/2/2009 21:43'!
onAttachDo: aBlock
	"The aBlock will be evaluated if the phidget was attached."

	processBlocks at: #Attach put: aBlock.! !

!PhdCrPhidget methodsFor: 'as yet unclassified' stamp: 'pt 5/2/2009 21:43'!
onDetachDo: aBlock
	"The aBlock will be evaluated if the phidget was detached."

	processBlocks at: #Detach put: aBlock.! !

!PhdCrPhidget methodsFor: 'as yet unclassified' stamp: 'pt 5/2/2009 21:45'!
onErrorDo: aBlock
	"The aBlock will be evaluated if an error occoured with the phidget."

	processBlocks at: #Error put: aBlock.! !

!PhdCrPhidget methodsFor: 'as yet unclassified' stamp: 'pt 5/2/2009 22:10'!
primLastError
	"Returns the error code of the last error.
	Descriptions of the codes may be found on www.phidgets.com or phidget21.h on line 2250 ff."

	<primitive: 'primLastError' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrPhidget methodsFor: 'as yet unclassified' stamp: 'pt 5/2/2009 22:44'!
registerSemaphores
	"This is where subclasses should register the semaphores, which handle the callbacks, with:
	| semaIndex |
	semaIndex := Smalltalk registerExternalObject: ( semaphores at: #aSymbol ).
	self primOnPositionChange: servo SemaphoreIndex: semaIndex."

	self subclassResponsibility.! !

!PhdCrPhidget methodsFor: 'as yet unclassified' stamp: 'pt 5/2/2009 22:16'!
unregisterSemaphores
	"Unregister the semaphores which were previously registered with 'self registerSemaphores'."

	semaphores do: [ :sema |
		sema signal.
		Smalltalk unregisterExternalObject: sema ]! !

PhdCrPhidget subclass: #PhdCrServo
	instanceVariableNames: 'servo'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Core'!

!PhdCrServo commentStamp: 'pt 5/3/2009 15:04' prior: 0!
A PhdCrServo is a representation of a Phidget called 'Servo' (here: 'Servo board') in Squeak.
To use the Servo board, create a new instance of this class and open a connection to the hardware board with:
	servo := PhdCrServo new.
	servo openAny.
A block can be set to be evaluated every time the hardware board was attached, detached, an error occoured or the position of a motor was changed. For example:
	servo
		onAttachDo: [ Transcript show:'servo was attached!!;cr' ];
		onDetachDo: [ Transcript show:'servo was detached!!;cr' ];
		onPositionChangeDo: [ Transcript show:'A motor changed it's position!!' ].
Depending on which hardware is attached it is possible to attach one or more servo motors to the board. To get the number of available motors send 'motorCount'. Every motor can be accessed by an index adressing the motor. Valid indices are greater or equal than 1 and less or equal than the number of available motors (motorCount).
To set the position of a motor to a value, call 'setMotorPosition: aFloat at: index' (aFloat must be between minMotorPositionAt: index and maxMotorPositionAt: index !!).

IMPORTANT:
After using the servo it is important to close the connection to the hardware by sending 'close':
	servo close.

Instance Variables
	servo:		<ByteArray>

servo
	- a ByteArray that contains the Servo board handle for the C API. Every primitive (except the one where the handle is created) needs this ByteArray to let the C API know about which Servo board it is currently talking.
!

!PhdCrServo methodsFor: 'as yet unclassified' stamp: 'pt 5/3/2009 20:06'!
attached
	"Returns true if the Servo board is attached."
	
	( ( self primAttached: servo ) == 1 ) ifTrue: [ ^ true ].
	^ false! !

!PhdCrServo methodsFor: 'as yet unclassified' stamp: 'pt 5/2/2009 23:23'!
close
	"Close the connection to the Servo board."

	opened ifFalse: [^ self]. "already closed"
	self primClose: servo.
	opened := false.
	self unregisterSemaphores.! !

!PhdCrServo methodsFor: 'as yet unclassified' stamp: 'pt 5/3/2009 11:26'!
getEngagedAt: index 
	"Returns the engaged state of a motor. This is whether the motor is powered or not."
	(self indexIsInRange: index) ifFalse: [ ^ self ].
	(self 
		primGetEngaged: servo
		at: index - 1) == 1 ifTrue: [ ^ true ].
	^ false! !

!PhdCrServo methodsFor: 'as yet unclassified' stamp: 'pt 5/3/2009 11:26'!
indexIsInRange: index 
	"Returns true if 1<=index<=motorCount.
	Every time a primitive is used that needs the motor index, this method should be used to check if the index is in the correct range."
	index isInteger ifFalse: [ ^ false ].
	^ 1 <= index and: [ index <= self motorCount ]! !

!PhdCrServo methodsFor: 'as yet unclassified' stamp: 'pt 5/2/2009 23:23'!
initialize
	"Initialise the callbacks that are not already initialised in PhdCrPhidget.
	Then initialise the Servo board handle, which is used to call the primitives."

	super initialize.
	semaphores add: #PositionChange -> Semaphore new.
	processBlocks add: #PositionChange->[].
	servo := self primCreate.! !

!PhdCrServo methodsFor: 'as yet unclassified' stamp: 'pt 5/3/2009 11:26'!
maxMotorPositionAt: index 
	"Returns the max. allowed motor position for the motor attached at the given index."
	(self indexIsInRange: index) ifFalse: [ ^ 0 ].
	^ self 
		primGetMotorPositionMax: servo
		at: index - 1! !

!PhdCrServo methodsFor: 'as yet unclassified' stamp: 'pt 5/3/2009 11:26'!
minMotorPositionAt: index 
	"Returns the min. allowed motor position for the motor attached at the given index."
	(self indexIsInRange: index) ifFalse: [ ^ 0 ].
	^ self 
		primGetMotorPositionMin: servo
		at: index - 1! !

!PhdCrServo methodsFor: 'as yet unclassified' stamp: 'pt 5/7/2009 18:42'!
motorCount
	"Returns the number of motors that may be attached to the Servo board."
	
	self attached
		ifTrue: [ ^ self primNumMotors: servo ].
	^ 0! !

!PhdCrServo methodsFor: 'as yet unclassified' stamp: 'pt 5/3/2009 11:26'!
motorPositionAt: index 
	"Returns the current position of the motor attached at the given index."
	(self indexIsInRange: index) ifFalse: [ ^ 0 ].
	^ self 
		primGetMotorPosition: servo
		at: index - 1! !

!PhdCrServo methodsFor: 'as yet unclassified' stamp: 'pt 5/2/2009 21:43'!
onPositionChangeDo: aBlock
	"The aBlock will be evaluated if the position of a motor was changed."

	processBlocks at: #PositionChange put: aBlock.! !

!PhdCrServo methodsFor: 'as yet unclassified' stamp: 'pt 5/2/2009 23:24'!
openAny
	"Opens any Servo board."
	
	servo ifNil: [ self error: 'Could not open Servo Board!!'. ^ self ].
	self registerSemaphores.
	self primOpenAny: servo.
	opened := true.
	[ self callbackProcess: #Attach ] fork.
	[ self callbackProcess: #Detach ] fork.
	[ self callbackProcess: #Error ] fork.
	[ self callbackProcess: #PositionChange ] fork.! !

!PhdCrServo methodsFor: 'as yet unclassified' stamp: 'pt 5/2/2009 23:23'!
openSpecific: serialNumber
	"Opens the Servo board with the given serial number."
	
	servo ifNil: [ self error: 'Could not open Servo Board!!'. ^ self ].
	self registerSemaphores.
	self primOpen: servo withSerial: serialNumber.
	opened := true.
	[ self callbackProcess: #Attach ] fork.
	[ self callbackProcess: #Detach ] fork.
	[ self callbackProcess: #Error ] fork.
	[ self callbackProcess: #PositionChange ] fork.! !

!PhdCrServo methodsFor: 'as yet unclassified' stamp: 'pt 5/4/2009 15:31'!
positionIsInRange: position at: index 
	"The position is in range, if the index is in range and the position is between the min. and max. Position."
	(self indexIsInRange: index) ifFalse: [ ^ false ].
	(self maxMotorPositionAt: index) >= position ifFalse: [ ^ false ].
	(self minMotorPositionAt: index) <= position ifFalse: [ ^ false ].
	^ true! !

!PhdCrServo methodsFor: 'primitives' stamp: 'pt 5/2/2009 23:24'!
primAttached: phid
	"Return 1 if the Servo board ist attached - otherwise return 0."

	<primitive: 'primServoAttached' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrServo methodsFor: 'primitives' stamp: 'pt 5/2/2009 23:24'!
primClose: phid
	"Close the connection to the Servo board."

	<primitive: 'primServoClose' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrServo methodsFor: 'primitives' stamp: 'pt 5/2/2009 23:24'!
primCreate
	"Reset the Servo board (Semaphores and Handle)."

	<primitive: 'primServoCreate' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrServo methodsFor: 'primitives' stamp: 'pt 5/2/2009 22:21'!
primGetEngaged: phid at: index
	"Returns 1 if the motor on the given index is engaged - otherwise return 0.
	This is whether the motor is powered or not."

	<primitive: 'primServoGetEngaged' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrServo methodsFor: 'primitives' stamp: 'pt 5/2/2009 22:21'!
primGetMotorPosition: phid at: index
	"Returns a float containing the position of the motor at the given index."

	<primitive: 'primServoGetMotorPosition' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrServo methodsFor: 'primitives' stamp: 'pt 5/2/2009 22:21'!
primGetMotorPositionMax: phid at: index
	"Returns a float containing max. possible position of the motor at the given index."

	<primitive: 'primServoGetMotorPositionMax' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrServo methodsFor: 'primitives' stamp: 'pt 5/2/2009 22:21'!
primGetMotorPositionMin: phid at: index
	"Returns a float containing min. possible position of the motor at the given index."

	<primitive: 'primServoGetMotorPositionMin' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrServo methodsFor: 'primitives' stamp: 'pt 5/2/2009 23:24'!
primNumMotors: phid
	"Returns the number of motors that can be attached to the Servo board."

	<primitive: 'primServoNumMotors' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrServo methodsFor: 'primitives' stamp: 'lk 5/5/2009 17:16'!
primOnAttached: phid semaphoreIndex: semaIndex
	"registers the given Semaphore to be signaled when the servo board becomes attached."
	
	<primitive: 'primServoOnAttached' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrServo methodsFor: 'primitives' stamp: 'lk 5/5/2009 17:16'!
primOnDetached: phid semaphoreIndex: semaIndex
	"registers the given Semaphore to be signaled when the Servo board becomes detached."
	
	<primitive: 'primServoOnDetached' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrServo methodsFor: 'primitives' stamp: 'lk 5/5/2009 17:16'!
primOnError: phid semaphoreIndex: semaIndex
	"registers the given Semaphore to be signaled when a an Error occours. Use primLastError to get the last error code"
	
	<primitive: 'primServoOnError' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrServo methodsFor: 'primitives' stamp: 'lk 5/5/2009 17:16'!
primOnPositionChanged: phid semaphoreIndex: semaIndex
	"registers the given Semaphore to be signaled when a motor changes its position."
	
	<primitive: 'primServoOnPositionChanged' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrServo methodsFor: 'primitives' stamp: 'pt 5/2/2009 23:26'!
primOpen: phid withSerial: serial
	"Opens the Servo board with the given serialNumber."

	<primitive: 'primServoOpen' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrServo methodsFor: 'primitives' stamp: 'pt 5/2/2009 23:26'!
primOpenAny: phid
	"Opens any available Servo board."

	<primitive: 'primServoOpenAny' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrServo methodsFor: 'primitives' stamp: 'pt 4/28/2009 10:13'!
primSetEngaged: phid at: index to: value
	"Sets the engaged state of the motor at the given index to <value>. Value must be 0 or 1!!
	This is whether the motor is powered or not."

	<primitive: 'primServoSetEngaged' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrServo methodsFor: 'primitives' stamp: 'pt 4/28/2009 10:45'!
primSetMotorPosition: phid at: index to: aFloat
	"Set the position of the motor at the given index to aFloat. aFloat must be between primGetPositionMin and primGetPositionMax."

	<primitive: 'primServoSetMotorPosition' module: 'PhidgetPlugin'>
	^ nil! !

!PhdCrServo methodsFor: 'as yet unclassified' stamp: 'lk 5/7/2009 13:28'!
registerSemaphores
	"Register the semaphores, so that the PhidgetPlugin can signal them."

	| semaIndex |
	semaIndex := Smalltalk registerExternalObject: ( semaphores at:#PositionChange ).
	self primOnPositionChanged: servo semaphoreIndex: semaIndex.
	semaIndex := Smalltalk registerExternalObject: ( semaphores at:#Attach ).
	self primOnAttached: servo semaphoreIndex: semaIndex.
	semaIndex := Smalltalk registerExternalObject: ( semaphores at:#Detach ).
	self primOnDetached: servo semaphoreIndex: semaIndex.
	semaIndex := Smalltalk registerExternalObject: ( semaphores at:#Error ).
	self primOnError: servo semaphoreIndex: semaIndex.! !

!PhdCrServo methodsFor: 'as yet unclassified' stamp: 'pt 5/3/2009 11:26'!
setEngaged: aBoolean at: index 
	"Sets the engaged state of a motor. This is whether the motor is powered or not."
	| anInt |
	(self indexIsInRange: index) ifFalse: [ ^ self ].
	aBoolean 
		ifTrue: [ anInt := 1 ]
		ifFalse: [ anInt := 0 ].
	self 
		primSetEngaged: servo
		at: index - 1
		to: anInt! !

!PhdCrServo methodsFor: 'as yet unclassified' stamp: 'pt 5/4/2009 15:31'!
setMotorPosition: aValue at: index
	"Set the position of the motor attached at <index> to <aValue>.
	aValue should be a float betwenn the min. and max. motor position."

	| aFloat |
	aFloat := aValue asFloat.
	( self positionIsInRange: aFloat at: index ) ifFalse: [ ^ self ].
	^ self primSetMotorPosition: servo at: index - 1 to: aFloat! !

PartsBin subclass: #PhdETPartsBin
	instanceVariableNames: 'controlingMorph'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-EToys'!

!PhdETPartsBin class methodsFor: 'as yet unclassified' stamp: 'pt 5/7/2009 13:17'!
thumbnailForQuad: aQuint color: aColor phidget: aPhidget
	"Answer a thumbnail for a morph obtaining as per the quintuplet provided, creating the thumbnail if necessary.  If it is created afresh, it will also be cached at this time"

	| aThumbnail aSymbol formToThumbnail labeledItem |
	aSymbol := aQuint third.
	Thumbnails at: aSymbol ifPresent: [ :thumb | ^thumb ].
	formToThumbnail := aQuint at: 5 ifAbsent: [].
	formToThumbnail ifNil: [
		labeledItem := (Smalltalk at: aQuint first) perform: aQuint second with: aPhidget.
		formToThumbnail := labeledItem imageForm: 32 backgroundColor: aColor forRectangle: labeledItem fullBounds.
		formToThumbnail replaceColor: aColor withColor: Color transparent.
	].

	aThumbnail := Thumbnail new makeThumbnailFromForm: formToThumbnail.
	self cacheThumbnail: aThumbnail forSymbol: aSymbol.
	^ aThumbnail

"PartsBin initialize"! !

!PhdETPartsBin methodsFor: 'as yet unclassified' stamp: 'pt 5/7/2009 11:47'!
controlingMorph
	^ controlingMorph! !

!PhdETPartsBin methodsFor: 'as yet unclassified' stamp: 'pt 5/7/2009 11:48'!
controlingMorph: aMorph
	controlingMorph := aMorph.! !

!PhdETPartsBin methodsFor: 'as yet unclassified' stamp: 'pt 5/7/2009 13:18'!
listDirection: aListDirection quadList: quadList
	"Initialize the receiver to run horizontally or vertically, obtaining its elements from the list of tuples of the form:
		(<receiver> <selector> <label> <balloonHelp>)"

	| aButton aClass |
	self layoutPolicy: TableLayout new.
	self listDirection: aListDirection.
	self wrapCentering: #topLeft.
	self layoutInset: 2.
	self cellPositioning: #bottomCenter.

	aListDirection == #leftToRight
		ifTrue:
			[self vResizing: #rigid.
			self hResizing: #spaceFill.
			self wrapDirection: #topToBottom]
		ifFalse:
			[self hResizing: #rigid.
			self vResizing: #spaceFill.
			self wrapDirection: #leftToRight].
	quadList do:
		[:tuple |
			aClass := Smalltalk at: tuple first.
			aButton := IconicButton new initializeWithThumbnail: (self class thumbnailForQuad: tuple color: self color phidget: self phidget) withLabel: tuple third andColor: self color andSend: tuple second to: aClass with: self phidget.
			(tuple size > 3 and: [tuple fourth isEmptyOrNil not]) ifTrue:
				[aButton setBalloonText: tuple fourth].
 			self addMorphBack: aButton]! !

!PhdETPartsBin methodsFor: 'as yet unclassified' stamp: 'pt 5/7/2009 11:58'!
phidget
	^ controlingMorph phidget! !

BorderedMorph subclass: #PhdETInterfaceKit
	instanceVariableNames: 'ifKit'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-EToys'!

!PhdETInterfaceKit class methodsFor: 'as yet unclassified' stamp: 'pt 5/6/2009 15:54'!
additionsToViewerCategories
	"Answer a list of (<categoryName> <list of category specs>) pairs that characterize the phrases this kind of morph wishes to add to various Viewer categories."
	^ #(
		(InterfaceKit (
			(slot attached 'This is wether the an hardware Interface Kit is attached to the USB port or not.' Boolean readOnly Player getAttached unused unused)
		))
	)

! !

!PhdETInterfaceKit class methodsFor: 'as yet unclassified' stamp: 'pt 5/3/2009 19:18'!
descriptionForPartsBin
    ^ self partName:    'InterfaceKit'
        categories:     #('Phidgets')
        documentation:  'An Interface Kit'! !

!PhdETInterfaceKit methodsFor: 'as yet unclassified' stamp: 'pt 5/7/2009 13:23'!
delete
	ifKit close.
	super delete.! !

!PhdETInterfaceKit methodsFor: 'as yet unclassified' stamp: 'pt 5/6/2009 15:54'!
getAttached
	^ ifKit attached! !

!PhdETInterfaceKit methodsFor: 'as yet unclassified' stamp: 'pt 5/7/2009 09:14'!
getInput: index
	^ ifKit inputValue: index! !

!PhdETInterfaceKit methodsFor: 'as yet unclassified' stamp: 'pt 5/7/2009 09:14'!
getOutput: index
	^ ifKit outputValue: index! !

!PhdETInterfaceKit methodsFor: 'as yet unclassified' stamp: 'pt 5/7/2009 09:15'!
getSensor: index
	^ ifKit sensorValue: index! !

!PhdETInterfaceKit methodsFor: 'as yet unclassified' stamp: 'pt 5/7/2009 12:16'!
initializeToStandAlone
	
	| aBin quads oldResizing |
	super initialize.
	self
		useRoundedCorners;
		width: 280;
		height: 60;
		color: (Color r: 0.4 g: 0.7 b: 0.4).
	ifKit := PhdCrInterfaceKit new.
	
	aBin := (PhdETPartsBin newPartsBinWithOrientation: #leftToRight from: #())
		listDirection: #leftToRight;
		wrapDirection: #topToBottom;
		color: self color lighter lighter;
		setNameTo: 'Ports';
		dropEnabled: false;
		height: self height;
		width: self width;
		useRoundedCorners;
		controlingMorph: self;
		yourself.

	self addMorph: aBin.

	oldResizing := aBin vResizing.
	quads := OrderedCollection new.
	PhdETIfKitInputPort addPortDescriptorQuadsTo: quads.
	PhdETIfKitOutputPort addPortDescriptorQuadsTo: quads.
	PhdETIfKitSensorPort addPortDescriptorQuadsTo: quads.
	aBin
		listDirection: #leftToRight quadList: quads;
		vResizing: oldResizing;
		layoutChanged;
		fullBounds.
	
	ifKit openAny.! !

!PhdETInterfaceKit methodsFor: 'as yet unclassified' stamp: 'pt 5/7/2009 12:21'!
phidget
	^ self! !

!PhdETInterfaceKit methodsFor: 'as yet unclassified' stamp: 'pt 5/7/2009 09:15'!
setOutput: index to: aBoolean
	^ ifKit setOutputValue: aBoolean at: index! !

BorderedMorph subclass: #PhdETPort
	instanceVariableNames: 'Phidget portNumber numberString'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-EToys'!

PhdETPort subclass: #PhdETIfKitInputPort
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-EToys'!

!PhdETIfKitInputPort class methodsFor: 'as yet unclassified' stamp: 'pt 5/7/2009 14:59'!
additionsToViewerCategoryInputPort
	"Answer a list of (<categoryName> <list of category specs>) pairs that characterize the phrases this kind of morph wishes to add to various Viewer categories."
	
	^ #(
		InputPort (
			(slot Value 'The value at the digital Intput port on an Interface Kit.' Boolean readOnly Player getIfKitInputValue unused unused)
			(slot PortNumber 'The index of this digital Input port.' Number readWrite Player getPortNumber Player setPortNumber:)
		)
	)! !

!PhdETIfKitInputPort class methodsFor: 'as yet unclassified' stamp: 'pt 5/7/2009 08:32'!
portDescription

	^  self
		partName:		'Digital Input'
		categories:		#('Interface Kit')
		documentation:	'An port to a digital input of an Interface Kit. This port is bound to a port of the Interface Kit hardware. With me it is possible to read wether the port returns true or false.'.! !

!PhdETIfKitInputPort methodsFor: 'as yet unclassified' stamp: 'pt 5/7/2009 15:00'!
getIfKitInputValue
	^ Phidget getInput: portNumber! !

!PhdETIfKitInputPort methodsFor: 'as yet unclassified' stamp: 'pt 5/7/2009 12:39'!
initialize
	super initialize.
	self color: Color red lighter.! !

PhdETPort subclass: #PhdETIfKitOutputPort
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-EToys'!

!PhdETIfKitOutputPort class methodsFor: 'as yet unclassified' stamp: 'pt 5/7/2009 14:59'!
additionsToViewerCategoryOutputPort
	"Answer a list of (<categoryName> <list of category specs>) pairs that characterize the phrases this kind of morph wishes to add to various Viewer categories."

	^ #(
		OutputPort (
			(slot Value 'The value at the digital Output port on an Interface Kit.' Boolean readWrite Player getIfKitOutputValue Player setIfKitOutputValue:)
			(slot PortNumber 'The index of this digital Output port.' Number readWrite Player getPortNumber Player setPortNumber:)
		)
	)! !

!PhdETIfKitOutputPort class methodsFor: 'as yet unclassified' stamp: 'pt 5/7/2009 08:32'!
portDescription

	^  self
		partName:		'Digital Output'
		categories:		#('Interface Kit')
		documentation:	'An port to a digital output of an Interface Kit. This port is bound to a port of the Interface Kit hardware. With me it is possible to read wether the port returns true or false or set this value'.! !

!PhdETIfKitOutputPort methodsFor: 'as yet unclassified' stamp: 'pt 5/7/2009 15:00'!
getIfKitOutputValue
	^ Phidget getOutput: portNumber! !

!PhdETIfKitOutputPort methodsFor: 'as yet unclassified' stamp: 'pt 5/7/2009 12:39'!
initialize
	super initialize.
	self color: Color blue lighter.! !

!PhdETIfKitOutputPort methodsFor: 'as yet unclassified' stamp: 'pt 5/7/2009 15:00'!
setIfKitOutputValue: aBoolean
	^ Phidget setOutput: portNumber to: aBoolean.! !

PhdETPort subclass: #PhdETIfKitSensorPort
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-EToys'!

!PhdETIfKitSensorPort class methodsFor: 'as yet unclassified' stamp: 'pt 5/7/2009 15:00'!
additionsToViewerCategorySensorPort
	"Answer a list of (<categoryName> <list of category specs>) pairs that characterize the phrases this kind of morph wishes to add to various Viewer categories."

	^ #(
		SensorPort (
			(slot Value 'The value at the analog Sensor port on an Interface Kit.' Number readOnly Player getIfKitSensorValue unused unused)
			(slot SensorPortNumber 'The index of this analog Sensor port.' Number readWrite Player getPortNumber Player setPortNumber:)
		)
	)! !

!PhdETIfKitSensorPort class methodsFor: 'as yet unclassified' stamp: 'pt 5/7/2009 08:32'!
portDescription

	^  self
		partName:		'Analog Sensor'
		categories:		#('Interface Kit')
		documentation:	'An port to an analog sensor of an Interface Kit. This port is bound to a port of the Interface Kit hardware. With me it is possible to read which value an attached analog sensor returns. This value is between 0 and 1000.'.! !

!PhdETIfKitSensorPort methodsFor: 'as yet unclassified' stamp: 'pt 5/7/2009 15:00'!
getIfKitSensorValue
	^ Phidget getSensor: portNumber! !

!PhdETIfKitSensorPort methodsFor: 'as yet unclassified' stamp: 'pt 5/7/2009 12:39'!
initialize
	super initialize.
	self color: Color yellow lighter.! !

!PhdETPort class methodsFor: 'as yet unclassified' stamp: 'pt 5/7/2009 08:17'!
addPortDescriptorQuadsTo: aList
	"analogous to addPartsDescriptorQuadsTo: aList if: aBlock from the Morph class"
	
	| info |
	info := self portDescription.
	aList add:
		{info globalReceiverSymbol.
		info nativitySelector.
		info formalName.
		info documentation.
		info sampleImageFormOrNil}.! !

!PhdETPort class methodsFor: 'as yet unclassified' stamp: 'pt 5/7/2009 12:00'!
launchPartVia: aSelector label: aString phidget: aPhidget
	"Obtain a morph by sending aSelector to self, and attach it to the morphic hand.  This provides a general protocol for parts bins"

	| aMorph |
	aMorph := self perform: aSelector with: aPhidget.
	aMorph setNameTo: (ActiveWorld unusedMorphNameLike: aString).
	aMorph setProperty: #beFullyVisibleAfterDrop toValue: true.
	aMorph openInHand! !

!PhdETPort class methodsFor: 'as yet unclassified' stamp: 'pt 5/7/2009 10:54'!
newStandAlone: aPhidget
	"Answer an instance capable of standing by itself as a usable morph."

	^ self basicNew initializeToStandAlone: aPhidget! !

!PhdETPort class methodsFor: 'as yet unclassified' stamp: 'pt 5/7/2009 10:52'!
partName: aName categories: aList documentation: aDoc
	"Answer a DescriptionForPartsBin which will represent a launch of a new instance of my class via the #newStandAlone protocol sent to my class. Use the category-list and documentation provided"


	^ DescriptionForPartsBin new
		formalName: aName
		categoryList: aList
		documentation: aDoc
		globalReceiverSymbol: self name
		nativitySelector: #newStandAlone:! !

!PhdETPort class methodsFor: 'as yet unclassified' stamp: 'pt 5/7/2009 08:18'!
portDescription

	self subclassResponsibility.
	! !

!PhdETPort methodsFor: 'as yet unclassified' stamp: 'pt 5/7/2009 09:51'!
centerNumberString.
	numberString position: self center - ((numberString width / 2)@( numberString height / 2)).! !

!PhdETPort methodsFor: 'as yet unclassified' stamp: 'pt 5/7/2009 09:28'!
getPortNumber
	^ portNumber! !

!PhdETPort methodsFor: 'as yet unclassified' stamp: 'pt 5/7/2009 12:02'!
initialize

	super initialize.
	self useRoundedCorners.
	portNumber := 1.
	numberString := StringMorph new contents: portNumber asString.
	self centerNumberString.
	self addMorph: numberString.! !

!PhdETPort methodsFor: 'as yet unclassified' stamp: 'pt 5/7/2009 11:01'!
initializeToStandAlone: aPhidget
	self initialize.
	Phidget := aPhidget.! !

!PhdETPort methodsFor: 'as yet unclassified' stamp: 'pt 5/7/2009 09:52'!
setPortNumber: aNumber
	aNumber isInteger ifTrue: [
		portNumber := aNumber.
		numberString contents: portNumber asString.
		self centerNumberString ].! !

BorderedMorph subclass: #PhdLabConnector
	instanceVariableNames: 'connector type'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Lab'!

!PhdLabConnector methodsFor: 'mouse events' stamp: 'sp 5/7/2009 08:18'!
click: anEvent
	"ignore this event, wait for drag:"

	^ self! !

!PhdLabConnector methodsFor: 'accessing' stamp: 'sp 5/4/2009 21:12'!
connector

	^ connector! !

!PhdLabConnector methodsFor: 'accessing' stamp: 'sp 5/4/2009 21:13'!
connector: aPhdLbConnector

	connector := aPhdLbConnector.! !

!PhdLabConnector methodsFor: 'mouse events' stamp: 'sp 5/7/2009 08:46'!
doubleClick: anEvent
	"ignore this event, wait for drag:"

	^ self! !

!PhdLabConnector methodsFor: 'mouse events' stamp: 'sp 5/7/2009 09:39'!
drag: anEvent
	""
	
"	anEvent hand addEventListener: self."
	
	| wire |
	wire := ( PhdLabWire 
				from: self center 
				to: Cursor position 
				color: Color black 
				width: 2 ) openInWorld.
	
	anEvent hand addMorphBack: wire.! !

!PhdLabConnector methodsFor: 'mouse events' stamp: 'sp 5/7/2009 08:46'!
firstClickTimedOut: anEvent
	"ignore this event, wait for drag:"

	^ self! !

!PhdLabConnector methodsFor: 'mouse events' stamp: 'sp 5/7/2009 08:01'!
handlesMouseDown: anEvent

	^ true! !

!PhdLabConnector methodsFor: 'initialize' stamp: 'sp 5/4/2009 21:15'!
initialize

	super initialize.
	
	self
		connector: nil;
		type: #unknown.! !

!PhdLabConnector methodsFor: 'mouse events' stamp: 'sp 5/7/2009 08:45'!
mouseDown: anEvent
	"when mouseDown-event inform the hand to watch for a drag: message (do we have to catch the other events??-like click)"

	"catch if okToPickUp is ok??" "what happend when user doesn't drag?'"
	
	anEvent hand 
		waitForClicksOrDrag: self
		event: anEvent
		selectors: #(#click: #doubleClick: #firstClickTimedOut: #drag:)
		threshold: 3.! !

!PhdLabConnector methodsFor: 'mouse events' stamp: 'sp 5/4/2009 21:28'!
mouseUp: anEvent

	"self register: aConnector"  	" how to get? via wire?"
	
	self! !

!PhdLabConnector methodsFor: 'accessing' stamp: 'sp 5/4/2009 21:13'!
type

	^ type! !

!PhdLabConnector methodsFor: 'accessing' stamp: 'sp 5/4/2009 21:14'!
type: aSymbol
	"or we take two methods isInput and isOutput"

	type := aSymbol.! !

BorderedMorph subclass: #PhdLabInterfaceKit
	instanceVariableNames: 'ifKit'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Lab'!

BorderedMorph subclass: #PhdLabSlider
	instanceVariableNames: 'pick max min maxValue minValue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Lab'!

!PhdLabSlider methodsFor: 'as yet unclassified' stamp: 'sp 4/26/2009 13:00'!
initialize

	super initialize.
	self color: Color blue lighter;
		width: 50;
		height: 200.
	pick := BorderedMorph new.
	pick color: Color black;
		width: 40;
		height: 20.
	self addMorph: pick.
	pick
		left: ( self left + 5 );
		top: ( self top + 5 ).
	min := 0.
	max := ( self bottom - 5 ) - ( self top + 5 ) - pick height.
	minValue := 0.
	maxValue := 100.
	! !

!PhdLabSlider methodsFor: 'as yet unclassified' stamp: 'sp 4/26/2009 13:02'!
rawValue: aNumber
	"0 <= aNumber <= 1000"
	"pick top: self bottom - 5 - pick height - (aNumber/1000*max)."

	pick top: self top + 5 + ( ( 1000 - aNumber ) / 1000 * max ).! !

!PhdLabSlider methodsFor: 'as yet unclassified' stamp: 'sp 4/26/2009 13:26'!
setMin: aSmallNumber andMax: aBigNumber

	minValue := aSmallNumber.
	maxValue := aBigNumber.! !

!PhdLabSlider methodsFor: 'as yet unclassified' stamp: 'sp 4/26/2009 13:40'!
value

	^ ( pick top - self top - 5 ) * maxValue / minValue! !

SmartSyntaxInterpreterPlugin subclass: #PhidgetPlugin
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-VMMaker-Plugins'!

!PhidgetPlugin commentStamp: 'pt 5/3/2009 13:55' prior: 0!
A PhidgetPlugin is a plugin to enable the use of Phidgets (For more information about Phidgets see www.phidgets.com).

Instance Variables: - none -
!

!PhidgetPlugin class methodsFor: 'as yet unclassified' stamp: 'pt 4/6/2009 17:20'!
hasHeaderFile
	"If there is a single intrinsic header file to be associated with the plugin, here is where you want to flag"
	^true! !

!PhidgetPlugin class methodsFor: 'as yet unclassified' stamp: 'pt 4/18/2009 17:58'!
requiresCrossPlatformFiles
	"default is ok for most, any plugin needing platform specific files must say so"
	^true! !

!PhidgetPlugin class methodsFor: 'as yet unclassified' stamp: 'pt 4/18/2009 17:57'!
requiresPlatformFiles
	"this plugin requires platform specific files in order to work"
	^true! !

!PhidgetPlugin methodsFor: 'Interface Kit' stamp: 'lk 5/7/2009 13:42'!
ifKitValueOf: oop
	"Return a pointer to the first byte of the PhidgetIfKit record within the given Smalltalk bytes object, or nil if oop is not a PhidgetIfKit record."

	self returnTypeC: 'PhidgetIfKit *'.
	interpreterProxy success:
		( ( interpreterProxy isIntegerObject: oop ) not and:
		 [ ( interpreterProxy isBytes: oop ) and:
		 [ ( interpreterProxy slotSizeOf: oop ) = ( self cCode: 'sizeof(PhidgetIfKit)' ) ] ] ).
	interpreterProxy failed ifTrue: [^ nil].
	^ self cCode: '(PhidgetIfKit *) (oop + 4)'! !

!PhidgetPlugin methodsFor: 'Plugin common' stamp: 'sp 4/26/2009 13:03'!
initialiseModule
	"Initialise the module"

	self export: true.
	^ self cCode: '(sqInt) phidgetPlugin_Initialise()' inSmalltalk: [true]! !

!PhidgetPlugin methodsFor: 'Phidget common' stamp: 'sp 4/26/2009 13:04'!
phidgetValueOf: oop
	"Return a pointer to the first byte of the Phidget record within the given Smalltalk bytes object, or nil if oop is not a Phidget record."

	self returnTypeC: 'Phidget *'.
	interpreterProxy success:
		( ( interpreterProxy isIntegerObject: oop ) not and:
		 [ ( interpreterProxy isBytes: oop ) and:
		 [ ( interpreterProxy slotSizeOf: oop ) = ( self cCode: 'sizeof(Phidget)' ) ] ] ).
	interpreterProxy failed ifTrue: [^ nil].
	^ self cCode: '(Phidget *) (oop + 4)'! !

!PhidgetPlugin methodsFor: 'Interface Kit' stamp: 'lk 5/3/2009 20:22'!
primIfKitAttached: ifKitHandle

	| phid |
	self var: #phid type: 'PhidgetIfKit *'.
	self primitive: 'primIfKitAttached' parameters: #(Oop ).
	
	phid := self ifKitValueOf: ifKitHandle.
	^ self interpreterProxy integerObjectOf: ( self cCode: 'phidget_Attached((Phidget *)phid)' )! !

!PhidgetPlugin methodsFor: 'Interface Kit' stamp: 'lk 5/3/2009 20:22'!
primIfKitClose: phidHandle

	| phid |
	self var: #phid type: 'PhidgetIfKit *'.
	self primitive: 'primIfKitClose' parameters: #(Oop ).
	
	phid := self ifKitValueOf: phidHandle.
	^ self cCode: 'phidgetIfKit_Close(phid)'! !

!PhidgetPlugin methodsFor: 'Interface Kit' stamp: 'lk 5/3/2009 19:45'!
primIfKitCreate

	| phidOop phid |
	self var: #phid type: 'PhidgetIfKit *'.
	self primitive: 'primIfKitCreate' parameters: #( ).
	
	phidOop := interpreterProxy instantiateClass: interpreterProxy 
		classByteArray indexableSize: ( self cCode: 'sizeof(PhidgetIfKit)' ).
	phid := self ifKitValueOf: phidOop.
	interpreterProxy failed ifFalse: [ self cCode: 'phidgetIfKit_ResetSemaphores(phid)' ].
	^ phidOop! !

!PhidgetPlugin methodsFor: 'Interface Kit' stamp: 'lk 5/7/2009 13:54'!
primIfKitGetSensorChangeTrigger: phidHandle at: index

	| phid |
	self var: #phid type: 'PhidgetIfKit *'.
	self primitive: 'primIfKitGetSensorChangeTrigger' parameters: #(Oop SmallInteger).
	
	phid := self ifKitValueOf: phidHandle.
	^ self interpreterProxy integerObjectOf: ( self cCode: 'phidgetIfKit_GetSensorChangeTrigger(phid, (int)index)' )! !

!PhidgetPlugin methodsFor: 'Interface Kit' stamp: 'lk 5/7/2009 13:40'!
primIfKitInputValue: phidHandle at: index

	| phid |
	self var: #phid type: 'PhidgetIfKit *'.
	self primitive: 'primIfKitInputValue' parameters: #(Oop SmallInteger ).
	
	phid := self ifKitValueOf: phidHandle.
	^ self interpreterProxy integerObjectOf: ( self cCode: 'phidgetIfKit_GetInputState(phid, (int)index)' )! !

!PhidgetPlugin methodsFor: 'Interface Kit' stamp: 'lk 5/3/2009 20:23'!
primIfKitNumInputs: phidHandle

	| phid |
	self var: #phid type: 'PhidgetIfKit *'.
	self primitive: 'primIfKitNumInputs' parameters: #(Oop ).
	
	phid := self ifKitValueOf: phidHandle.
	^ self interpreterProxy integerObjectOf: ( self cCode: 'phidgetIfKit_NumInputs(phid)' )! !

!PhidgetPlugin methodsFor: 'Interface Kit' stamp: 'lk 5/3/2009 20:23'!
primIfKitNumOutputs: phidHandle

	| phid |
	self var: #phid type: 'PhidgetIfKit *'.
	self primitive: 'primIfKitNumOutputs' parameters: #(Oop ).
	
	phid := self ifKitValueOf: phidHandle.
	^ self interpreterProxy integerObjectOf: ( self cCode: 'phidgetIfKit_NumOutputs(phid)' )! !

!PhidgetPlugin methodsFor: 'Interface Kit' stamp: 'lk 5/3/2009 19:45'!
primIfKitNumSensors: phidHandle

	| phid |
	self var: #phid type: 'PhidgetIfKit *'.
	self primitive: 'primIfKitNumSensors' parameters: #(Oop ).
	
	phid := self ifKitValueOf: phidHandle.
	^ self interpreterProxy integerObjectOf: ( self cCode: 'phidgetIfKit_NumSensors(phid)' )! !

!PhidgetPlugin methodsFor: 'Interface Kit' stamp: 'lk 5/5/2009 16:56'!
primIfKitOnAttached: phidHandle semaphoreIndex: semaIndex

	| phid |
	self var: #phid type: 'PhidgetIfKit *'.
	self primitive: 'primIfKitOnAttached' parameters: #(Oop SmallInteger ).
	
	phid := self ifKitValueOf: phidHandle.
	^ self interpreterProxy integerObjectOf: ( self cCode: 'phidget_SetSemaIndexAttached((Phidget *)phid, (int)semaIndex)' )! !

!PhidgetPlugin methodsFor: 'Interface Kit' stamp: 'lk 5/5/2009 16:57'!
primIfKitOnDetached: phidHandle semaphoreIndex: semaIndex

	| phid |
	self var: #phid type: 'PhidgetIfKit *'.
	self primitive: 'primIfKitOnDetached' parameters: #(Oop SmallInteger ).

	phid := self ifKitValueOf: phidHandle.
	^ self interpreterProxy integerObjectOf: ( self cCode: 'phidget_SetSemaIndexDetached((Phidget *)phid, (int)semaIndex)' )! !

!PhidgetPlugin methodsFor: 'Interface Kit' stamp: 'lk 5/5/2009 16:57'!
primIfKitOnError: phidHandle semaphoreIndex: semaIndex

	| phid |
	self var: #phid type: 'PhidgetIfKit *'.
	self primitive: 'primIfKitOnError' parameters: #(Oop SmallInteger ).
	
	phid := self ifKitValueOf: phidHandle.
	^ self interpreterProxy integerObjectOf: ( self cCode: 'phidget_SetSemaIndexError((Phidget *)phid, (int)semaIndex)' )! !

!PhidgetPlugin methodsFor: 'Interface Kit' stamp: 'lk 5/5/2009 16:57'!
primIfKitOnInputChanged: phidHandle semaphoreIndex: semaIndex

	| phid |
	self var: #phid type: 'PhidgetIfKit *'.
	self primitive: 'primIfKitOnInputChanged' parameters: #(Oop SmallInteger ).
	
	phid := self ifKitValueOf: phidHandle.
	^ self interpreterProxy integerObjectOf: ( self cCode: 'phidgetIfKit_SetSemaIndexInputChanged(phid, (int)semaIndex)' )! !

!PhidgetPlugin methodsFor: 'Interface Kit' stamp: 'lk 5/5/2009 16:57'!
primIfKitOnOutputChanged: phidHandle semaphoreIndex: semaIndex

	| phid |
	self var: #phid type: 'PhidgetIfKit *'.
	self primitive: 'primIfKitOnOutputChanged' parameters: #(Oop SmallInteger ).
	
	phid := self ifKitValueOf: phidHandle.
	^ self interpreterProxy integerObjectOf: ( self cCode: 'phidgetIfKit_SetSemaIndexOutputChanged(phid, (int)semaIndex)' )! !

!PhidgetPlugin methodsFor: 'Interface Kit' stamp: 'lk 5/5/2009 16:58'!
primIfKitOnSensorChanged: phidHandle semaphoreIndex: semaIndex

	| phid |
	self var: #phid type: 'PhidgetIfKit *'.
	self primitive: 'primIfKitOnSensorChanged' parameters: #(Oop SmallInteger ).
	
	phid := self ifKitValueOf: phidHandle.
	^ self interpreterProxy integerObjectOf: (self cCode: 'phidgetIfKit_SetSemaIndexSensorChanged(phid, (int)semaIndex)')! !

!PhidgetPlugin methodsFor: 'Interface Kit' stamp: 'lk 5/3/2009 19:48'!
primIfKitOpen: phidOop withSerial: serial

	| phid |
	self var: #phid type: 'PhidgetIfKit *'.
	self primitive: 'primIfKitOpen' parameters: #(Oop SmallInteger ).
	
	phid := self ifKitValueOf: phidOop.
	interpreterProxy failed ifFalse: [ self cCode: 'phidgetIfKit_Open(phid, (int)serial)' ].
	^ phidOop! !

!PhidgetPlugin methodsFor: 'Interface Kit' stamp: 'lk 5/3/2009 19:48'!
primIfKitOpenAny: phidOop

	| phid |
	self var: #phid type: 'PhidgetIfKit *'.
	self primitive: 'primIfKitOpenAny' parameters: #(Oop ).
	
	phid := self ifKitValueOf: phidOop.
	interpreterProxy failed ifFalse: [ self cCode: 'phidgetIfKit_OpenAny(phid)' ].
	^ phidOop! !

!PhidgetPlugin methodsFor: 'Interface Kit' stamp: 'lk 5/7/2009 13:41'!
primIfKitOutputValue: phidHandle at: index

	| phid |
	self var: #phid type: 'PhidgetIfKit *'.
	self primitive: 'primIfKitOutputValue' parameters: #(Oop SmallInteger ).
	
	phid := self ifKitValueOf: phidHandle.
	^ self interpreterProxy integerObjectOf: ( self cCode: 'phidgetIfKit_GetOutputState(phid, (int)index)' )! !

!PhidgetPlugin methodsFor: 'Interface Kit' stamp: 'lk 5/3/2009 19:48'!
primIfKitSensorValue: phidHandle at: index

	| phid |
	self var: #phid type: 'PhidgetIfKit *'.
	self primitive: 'primIfKitSensorValue' parameters: #(Oop SmallInteger ).
	
	phid := self ifKitValueOf: phidHandle.
	^ self interpreterProxy integerObjectOf: ( self cCode: 'phidgetIfKit_GetSensorValue(phid, (int)index)' )! !

!PhidgetPlugin methodsFor: 'Interface Kit' stamp: 'lk 5/7/2009 13:42'!
primIfKitSetOutput: phidHandle at: index to: aNumber

	| phid |
	self var: #phid type: 'PhidgetIfKit *'.
	self primitive: 'primIfKitSetOutput' parameters: #(Oop SmallInteger SmallInteger).
	
	phid := self ifKitValueOf: phidHandle.
	^ self interpreterProxy integerObjectOf: ( self cCode: 'phidgetIfKit_SetOutputState(phid, (int)index, (int)aNumber)' )! !

!PhidgetPlugin methodsFor: 'Interface Kit' stamp: 'lk 5/7/2009 13:52'!
primIfKitSetSensorChangeTrigger: phidHandle at: index to: aNumber

	| phid |
	self var: #phid type: 'PhidgetIfKit *'.
	self primitive: 'primIfKitSetSensorChangeTrigger' parameters: #(#Oop #SmallInteger #SmallInteger ).
	phid := self ifKitValueOf: phidHandle.
	^ self interpreterProxy
		integerObjectOf: (self cCode: 'phidgetIfKit_SetSensorChangeTrigger(phid, (int)index, (int)aNumber)')! !

!PhidgetPlugin methodsFor: 'Phidget common' stamp: 'sp 4/26/2009 13:20'!
primLastError

	self primitive: 'primLastError' parameters: #( ).
	^ self interpreterProxy integerObjectOf: ( self cCode: 'phidget_LastError()' )! !

!PhidgetPlugin methodsFor: 'Servo' stamp: 'lk 5/3/2009 20:24'!
primServoAttached: servoHandle

	| phid |
	self var: #phid type: 'PhidgetServo *'.
	self primitive: 'primServoAttached' parameters: #(Oop ).
	
	phid := self servoValueOf: servoHandle.
	^ self interpreterProxy integerObjectOf: ( self cCode: 'phidget_Attached((Phidget *)phid)' )! !

!PhidgetPlugin methodsFor: 'Servo' stamp: 'lk 5/3/2009 20:24'!
primServoClose: phidHandle

	| phid |
	self var: #phid type: 'PhidgetServo *'.
	self primitive: 'primServoClose' parameters: #(Oop ).
	
	phid := self servoValueOf: phidHandle.
	^ self cCode: 'phidgetServo_Close(phid)'! !

!PhidgetPlugin methodsFor: 'Servo' stamp: 'lk 5/3/2009 19:49'!
primServoCreate

	| phidOop phid |
	self var: #phid type: 'PhidgetServo *'.
	self primitive: 'primServoCreate' parameters: #( ).
	
	phidOop := interpreterProxy instantiateClass: interpreterProxy 
		classByteArray indexableSize: ( self cCode: 'sizeof(PhidgetServo)' ).
	phid := self servoValueOf: phidOop.
	interpreterProxy failed ifFalse: [ self cCode: 'phidgetServo_ResetSemaphores(phid)' ].
	^ phidOop! !

!PhidgetPlugin methodsFor: 'Servo' stamp: 'lk 5/3/2009 20:24'!
primServoGetEngaged: servoHandle at: index

	| phid |
	self var: #phid type: 'PhidgetServo *'.
	self primitive: 'primServoGetEngaged' parameters: #(Oop SmallInteger).
	
	phid := self servoValueOf: servoHandle.
	^ self interpreterProxy integerObjectOf: ( self cCode: 'phidgetServo_GetEngaged(phid, (int)index)' )! !

!PhidgetPlugin methodsFor: 'Servo' stamp: 'lk 5/3/2009 20:15'!
primServoGetPosition: phidHandle at: index

	| phid |
	self var: #phid type: 'PhidgetServo *'.
	self primitive: 'primServoGetPosition' parameters: #(Oop SmallInteger ).
	
	phid := self servoValueOf: phidHandle.
	^ interpreterProxy floatObjectOf: ( self cCode: 'phidgetServo_GetPosition(phid, (int)index)' )! !

!PhidgetPlugin methodsFor: 'Servo' stamp: 'lk 5/3/2009 20:15'!
primServoGetPositionMax: phidHandle at: index

	| phid |
	self var: #phid type: 'PhidgetServo *'.
	self primitive: 'primServoGetPositionMax' parameters: #(Oop SmallInteger ).
	
	phid := self servoValueOf: phidHandle.
	^ interpreterProxy floatObjectOf: ( self cCode: 'phidgetServo_GetPositionMax(phid, (int)index)' )! !

!PhidgetPlugin methodsFor: 'Servo' stamp: 'lk 5/3/2009 20:15'!
primServoGetPositionMin: phidHandle at: index

	| phid |
	self var: #phid type: 'PhidgetServo *'.
	self primitive: 'primServoGetPositionMin' parameters: #(Oop SmallInteger ).
	
	phid := self servoValueOf: phidHandle.
	^ interpreterProxy floatObjectOf: ( self cCode: 'phidgetServo_GetPositionMin(phid, (int)index)' )! !

!PhidgetPlugin methodsFor: 'Servo' stamp: 'lk 5/3/2009 20:21'!
primServoNumMotors: servoHandle

	| phid |
	self var: #phid type: 'PhidgetServo *'.
	self primitive: 'primServoNumMotors' parameters: #(Oop ).
	
	phid := self servoValueOf: servoHandle.
	^ self interpreterProxy integerObjectOf: ( self cCode: 'phidgetServo_NumMotors(phid)' )! !

!PhidgetPlugin methodsFor: 'Servo' stamp: 'lk 5/5/2009 17:04'!
primServoOnAttached: phidHandle semaphoreIndex: semaIndex 

	| phid |
	self var: #phid type: 'PhidgetServo *'.
	self primitive: 'primServoOnAttached' parameters: #(#Oop #SmallInteger ).
	
	phid := self servoValueOf: phidHandle.
	^ self interpreterProxy integerObjectOf: (self cCode: 'phidget_SetSemaIndexAttached((Phidget *)phid, (int)semaIndex)')! !

!PhidgetPlugin methodsFor: 'Servo' stamp: 'lk 5/5/2009 17:04'!
primServoOnDetached: phidHandle semaphoreIndex: semaIndex

	| phid |
	self var: #phid type: 'PhidgetServo *'.
	self primitive: 'primServoOnDetached' parameters: #(Oop SmallInteger ).

	phid := self servoValueOf: phidHandle.
	^ self interpreterProxy integerObjectOf: ( self cCode: 'phidget_SetSemaIndexDetached((Phidget *)phid, (int)semaIndex)' )! !

!PhidgetPlugin methodsFor: 'Servo' stamp: 'lk 5/5/2009 17:04'!
primServoOnError: phidHandle semaphoreIndex: semaIndex

	| phid |
	self var: #phid type: 'PhidgetServo *'.
	self primitive: 'primServoOnError' parameters: #(Oop SmallInteger ).
	
	phid := self servoValueOf: phidHandle.
	^ self interpreterProxy integerObjectOf: ( self cCode: 'phidget_SetSemaIndexError((Phidget *)phid, (int)semaIndex)' )! !

!PhidgetPlugin methodsFor: 'Servo' stamp: 'lk 5/5/2009 17:04'!
primServoOnPositionChanged: phidHandle semaphoreIndex: semaIndex

	| phid |
	self var: #phid type: 'PhidgetServo *'.
	self primitive: 'primServoOnPositionChanged' parameters: #(Oop SmallInteger ).
	
	phid := self servoValueOf: phidHandle.
	^ self interpreterProxy integerObjectOf: ( self cCode: 'phidgetServo_SetSemaIndexPositionChanged(phid, (int)semaIndex)' )! !

!PhidgetPlugin methodsFor: 'Servo' stamp: 'lk 5/3/2009 19:51'!
primServoOpen: phidOop withSerial: serial

	| phid |
	self var: #phid type: 'PhidgetServo *'.
	self primitive: 'primServoOpen' parameters: #(Oop SmallInteger ).
	
	phid := self servoValueOf: phidOop.
	interpreterProxy failed ifFalse: [ self cCode: 'phidgetServo_Open(phid, (int)serial)' ].
	^ phidOop! !

!PhidgetPlugin methodsFor: 'Servo' stamp: 'lk 5/3/2009 19:51'!
primServoOpenAny: phidOop

	| phid |
	self var: #phid type: 'PhidgetServo *'.
	self primitive: 'primServoOpenAny' parameters: #(Oop ).
	
	phid := self servoValueOf: phidOop.
	interpreterProxy failed ifFalse: [ self cCode: 'phidgetServo_OpenAny(phid)' ].
	^ phidOop! !

!PhidgetPlugin methodsFor: 'Servo' stamp: 'lk 5/3/2009 19:51'!
primServoSetEngaged: servoHandle at: index to: value

	| phid |
	self var: #phid type: 'PhidgetServo *'.
	self primitive: 'primServoSetEngaged' parameters: #(Oop SmallInteger SmallInteger ).
	
	interpreterProxy success: (value<=1 and: [ value >=0 ]).
	interpreterProxy failed ifTrue: [^ nil].
	phid := self servoValueOf: servoHandle.
	^ self interpreterProxy integerObjectOf: ( self cCode: 'phidgetServo_SetEngaged(phid, (int)index, value)' )! !

!PhidgetPlugin methodsFor: 'Servo' stamp: 'lk 5/3/2009 19:51'!
primServoSetMotorPosition: phidHandle at: index to: value

	| phid floatValue |
	self var: #phid type: 'PhidgetServo *'.
	self var: #floatValue type: 'double'.
	self primitive: 'primServoSetMotorPosition' parameters: #(Oop SmallInteger Oop ).
	
	phid := self servoValueOf: phidHandle.
	floatValue := interpreterProxy floatValueOf: value.
	^ self interpreterProxy integerObjectOf: ( self cCode: 'phidgetServo_SetPosition(phid, (int)index, floatValue)' )! !

!PhidgetPlugin methodsFor: 'Servo' stamp: 'lk 5/3/2009 20:21'!
primServoSetPosition: phidHandle at: index to: value

	| phid floatValue |
	self var: #phid type: 'PhidgetServo *'.
	self var: #floatValue type: 'double'.
	self primitive: 'primServoSetPosition' parameters: #(Oop SmallInteger Oop ).
	
	phid := self servoValueOf: phidHandle.
	floatValue := interpreterProxy floatValueOf: value.
	^ self interpreterProxy integerObjectOf: ( self cCode: 'phidgetServo_SetPosition(phid, (int)index, floatValue)' )! !

!PhidgetPlugin methodsFor: 'Servo' stamp: 'lk 5/3/2009 19:52'!
servoValueOf: oop
	"Return a pointer to the first byte of the Phidget_servo record within the given Smalltalk bytes object, or nil if oop is not a Phidget_servo record."

	self returnTypeC: 'PhidgetServo *'.
	interpreterProxy success:
		( ( interpreterProxy isIntegerObject: oop ) not and:
		 [ ( interpreterProxy isBytes: oop ) and:
		 [ ( interpreterProxy slotSizeOf: oop ) = ( self cCode: 'sizeof(PhidgetServo)' ) ] ] ).
	interpreterProxy failed ifTrue: [^ nil].
	^ self cCode: '(PhidgetServo *) (oop + 4)'! !

!PhidgetPlugin methodsFor: 'Plugin common' stamp: 'sp 4/26/2009 13:20'!
shutdownModule
	"Shutdown the module"
	
	self export: true.
	^ self cCode: '(sqInt) phidgetPlugin_Shutdown()' inSmalltalk: [true]! !

LineMorph subclass: #PhdLabWire
	instanceVariableNames: 'startpoint endpoint'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Lab'!

!PhdLabWire methodsFor: 'accessing' stamp: 'sp 5/7/2009 09:40'!
endpoint

	^ endpoint! !

!PhdLabWire methodsFor: 'accessing' stamp: 'sp 5/7/2009 09:43'!
setEndpoint: aPoint

	endpoint := aPoint.
	"self set point"! !

!PhdLabWire methodsFor: 'accessing' stamp: 'sp 5/7/2009 09:42'!
setStartpoint: aPoint

	startpoint := aPoint.
	"self set point"! !

!PhdLabWire methodsFor: 'accessing' stamp: 'sp 5/7/2009 09:40'!
startpoint

	^ startpoint! !

!Player methodsFor: '*Phidget' stamp: 'pt 5/6/2009 15:55'!
getAttached
	"Answers wether an Interface Kit is attached or not."

	^ self costume getAttached! !

!Player methodsFor: '*Phidget' stamp: 'pt 5/7/2009 14:56'!
getIfKitInputValue
	"Answer the value of a Phidget Port"

	^ self costume getIfKitInputValue! !

!Player methodsFor: '*Phidget' stamp: 'pt 5/7/2009 14:57'!
getIfKitOutputValue
	"Answer the value of a Phidget Port"

	^ self costume getIfKitOutputValue! !

!Player methodsFor: '*Phidget' stamp: 'pt 5/7/2009 14:56'!
getIfKitSensorValue
	"Answer the value of a Phidget Port"

	^ self costume getIfKitSensorValue! !

!Player methodsFor: '*Phidget' stamp: 'pt 5/7/2009 09:27'!
getPortNumber
	"Answers the index of a Phidget port."

	^ self costume getPortNumber! !

!Player methodsFor: '*Phidget' stamp: 'pt 5/7/2009 14:57'!
setIfKitOutputValue: value
	"Set the value of a Phidget Port"

	^ self costume setIfKitOutputValue: value! !

!Player methodsFor: '*Phidget' stamp: 'pt 5/7/2009 09:27'!
setPortNumber: aNumber
	"Answers the index of a Phidget port."

	^ self costume setPortNumber: aNumber! !

TestCase subclass: #PhidgetPluginTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Tests'!

!PhidgetPluginTest methodsFor: 'as yet unclassified' stamp: 'sp 4/26/2009 13:21'!
testTest

	"Hier soll ohne Vorbedingungen getestet werden wie getestet wird ;)"
	^ true! !
